/*
@memo   
@name Prototype
@author .. , .. 
@version 1.0
*/

#include "pardbms.h"	/*   */
#include "errcode.h"	/*   */
#include "common.h"		/*     */
#include "dbconfig.h"	/*     */
#include "dbgen.h"		/*    */
#include "compiler.h"	/*   */
#include "lineplan.h"	/*    */
#include "parallel.h"	/*   */
#include "agentmgr.h"	/*    */
#include "engine.h"		/*   */
#include "filemgr.h"	/*   */
#include "msgmgr.h"		/*   */
#include "debug.h"		/*    */
#include <stdlib.h>
#include <assert.h>
#include <stdio.h>

/*
   
@name Prototype_Variables
@memo  
*/

/*@{*/

/*
@memo  
*/
static int initflag=FALSE;

/*
@memo   
*/
static int whoami;

/*
@memo  (,  ,  ,  ,  )
*/
static treenode_t * QUERY_TREE;

/*
@memo  
*/
static treenode_t * PARALLEL_AGENT;

/*@}*/

/*@{*/

/*
   
@name Prototype_Functions
@memo  
*/

/*@{*/

/*
          .
      .
@memo  
@return SUCCESS       
@see dbms_DoneDBMS dbms_RunDBMS
*/
int dbms_InitDBMS()
{
	int i, res;

	if (initflag)
		return SUCCESS;
	initflag=TRUE;
	res=mm_InitMessageManager();
	assert(res==SUCCESS);
	whoami=mm_WhoAmI();	
	assert(whoami>=0 && whoami<NODES_IN_CLUSTER);

	DBG_INIT(whoami);
	DBG_VISIBLE_ALL;

	START("dbms_InitDBMS");

	res=db_InitDatabase(whoami, NULL);
	assert(res==SUCCESS);
	res=fm_InitFileManager(whoami);
	assert(res==SUCCESS);
	for (i=0; i<RELATIONS_IN_DATABASE; i++) {
		res=fm_OpenRelation(i);
		assert(res==SUCCESS);
	}
	assert(res==SUCCESS);
	res=cp_InitCompiler(whoami);
	assert(res==SUCCESS);
	res=lp_InitPlanGenerator(whoami);
	assert(res==SUCCESS);	
	res=pl_InitParallelizer(whoami);
	assert(res==SUCCESS);
	res=am_InitAgentManager(whoami);
	assert(res==SUCCESS);
	res=en_InitEngine(whoami);
	assert(res==SUCCESS);

	STOP("dbms_InitDBMS");
	return SUCCESS;
}

/*
      .
@memo  
@return SUCCESS       
@see dbms_InitDBMS dbms_DoneDBMS
*/
int dbms_RunDBMS()
{
	int res;

	START("dbms_RunDBMS");
	if (!initflag)
		return ENOINIT;
	QUERY_TREE=cp_CompileQuery(QUERY_FNAME);
	assert(QUERY_TREE!=NULL);
	res=lp_GeneratePlan(QUERY_TREE);
	assert(res==SUCCESS);
	res=pl_ParallelizeQuery(QUERY_TREE);
	assert(res==SUCCESS);
	
	PARALLEL_AGENT=am_MakeAgent(QUERY_TREE);
	assert(PARALLEL_AGENT!=NULL);
	WATCH(("agentstr==%s", agentstr));
	
	res=en_ExecuteQuery(PARALLEL_AGENT);
	assert(res==SUCCESS);
	STOP("dbms_RunDBMS");
	return SUCCESS;
}

/*
          .
@memo  
@return SUCCESS       
@see dbms_InitDBMS dbms_RunDBMS
*/
int dbms_DoneDBMS(void)
{
	int i, res;

	START("dbms_DoneDBMS");
	if (!initflag)
		return ENOINIT;	
	for (i=0; i<RELATIONS_IN_DATABASE; i++) {
		res=fm_CloseRelation(i);
		assert(res==SUCCESS);
	}
	mm_DoneMessageManager();
	STOP("dbms_DoneDBMS");
	return SUCCESS;
}

/*@}*/

/*@}*/
